/*
 * Copyright (C) 2010 Canon IT Solutions Inc. All Rights Reserved.
 */

/**********************************************************************
* Modified by
* (C) NEC CASIO Mobile Communications, Ltd. 2011
**********************************************************************/

#ifndef ANDROID_EMOJI_ANIMATEDEMOJIFONT_H
#define ANDROID_EMOJI_ANIMATEDEMOJIFONT_H

#include "SkScalar.h"

class SkBitmap;
class SkCanvas;
class SkPaint;


namespace android {

class IAnimatedEmojiCallback;
class IAnimatedEmojiController;
class AnimatedEmojiController;

class AnimatedEmojiFont {
    public:
        /** Returns true if the underlying emoji font mechanism is available.
         */
        static bool IsAvailable();

        /** Return index for the corresponding index to the emoji table, or 0
            if there is no matching emoji form.
         */
        static uint16_t UnicharToGlyph(int32_t unichar);

        /** Returns true if the specified glyph is in the emoji range, i.e. was
            returned by UnicharToGlyph or UTF16ToGlyph.
         */
        static bool IsEmojiGlyph(uint16_t index) {
            return index >= kGlyphBase;
        }

        /** Returns the advance width for the specified emoji form.
         */
        static SkScalar GetAdvanceWidth(uint16_t index, const SkPaint& paint);

        /** Draw the specified emoji form, given the x,y origin of the text
            version. The paint is the one associated with the text that has
            the emoji in it.
         */
        static void Draw(SkCanvas*, uint16_t index, SkScalar x, SkScalar y,
                         const SkPaint& paint, bool useAnimation);

        static IAnimatedEmojiController *CreateAnimatedEmojiController(IAnimatedEmojiCallback *callback);

        static void DestroyAnimatedEmojiController(IAnimatedEmojiController *controller);

        static void StartAnimationIfNeeded(IAnimatedEmojiController *controller, uint16_t index);

        static void StopAnimation(IAnimatedEmojiController *controller);

        static bool IsAnimation(const IAnimatedEmojiController *controller);

        static void SetUseAnimation(IAnimatedEmojiController *controller, bool useAnimation);

    private:
        enum {
            /*  this is our internal trick to embedded private emoji glyph IDs
                along side normal glyphs IDs that come from real fonts. The
                assumption is that normal fonts never will report a glyph ID
                above 20K or 30K, so 64000 should always be a safe starting
                index. We also assume the the number of emoji will not overflow
                16bits starting at 64000 i.e. 65535 - 64000 > total emoji count
             */
            kGlyphBase = 64000
        };

        friend class AnimatedEmojiController;


        static uint64_t GetNextDuration();

        static int GetFrameIndex(uint16_t index);

        static int GetDefaultFrameIndex(uint16_t index);

        static int GetFrameCount(uint16_t index);

        static int GetBitmap(SkBitmap *bitmap, uint16_t index, int frameIndex, float fontSize);
        
        static int32_t GlyphToAndroidPua(uint16_t glyphID);

};

}   // namespace android

#endif  // ANDROID_EMOJI_ANIMATEDEMOJIFONT_H
